/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese.actions;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.border.BevelBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;

import com.simulogic.cese.CESE;
import com.simulogic.cese.ResourceManager;
import com.simulogic.cese.UserHome;
import com.simulogic.utils.ActionSupport;

/**
 * About action implementation
 */
public class AboutAction extends ActionSupport implements HyperlinkListener {

    private JDialog aboutBox;
    private JButton okButton;

    JEditorPane readmePanel = null;

    /** Creates new AboutAction */
    public AboutAction() {
        super(ResourceManager.getResource("About..."), ResourceManager
                .getResource("About"), ResourceManager
                .getResource("about_image"));
    }

    public void actionPerformed(ActionEvent evt) {
        createAboutDialog();
        aboutBox.setLocationRelativeTo(aboutBox.getOwner());
        aboutBox.setVisible(true);
        aboutBox.toFront();
    }

    private void createAboutDialog() {
        if (aboutBox == null) {
            aboutBox = new JDialog(CESE.getEnvironment(), ResourceManager
                    .getResource("About"), true);

            CESE.getEnvironment().getHelpBroker().enableHelpKey(
                    aboutBox.getRootPane(), "Dialogs.About", null);

            JPanel contentPanel = new JPanel(new BorderLayout());

            JPanel buttonPanel = new JPanel(new java.awt.FlowLayout(
                    java.awt.FlowLayout.RIGHT));

            okButton = (JButton) buttonPanel.add(new JButton(ResourceManager
                    .getResource("OK")));
            okButton.addActionListener(new java.awt.event.ActionListener() {

                public void actionPerformed(java.awt.event.ActionEvent e) {
                    aboutBox.setVisible(false);
                    aboutBox.dispose();
                }
            });

            JTabbedPane tabs = new JTabbedPane();
            tabs.addTab(ResourceManager.getResource("About"),
                    createProductPane());
            tabs.addTab(ResourceManager.getResource("System"), new JScrollPane(
                    createSystemPane()));
            tabs.addTab(ResourceManager.getResource("Readme"), new JScrollPane(
                    createReadmePane()));

            for (int i = 0; i < tabs.getTabCount(); i++)
                ((JComponent) tabs.getComponentAt(i)).setPreferredSize(tabs
                        .getComponentAt(0).getPreferredSize());

            contentPanel.add(tabs, BorderLayout.CENTER);
            contentPanel.add(buttonPanel, BorderLayout.SOUTH);
            aboutBox.getContentPane().add(contentPanel);

            aboutBox.pack();
            aboutBox.setResizable(false);
        }
        aboutBox.getRootPane().setDefaultButton(okButton);
    }

    private JPanel createProductPane() {
        JPanel productPanel = new JPanel(new BorderLayout());
        ImageIcon productPicture = new ImageIcon(ClassLoader
                .getSystemResource("com/simulogic/cese/images/heart.png"));
        productPanel.add(new JLabel(productPicture), BorderLayout.WEST);
        JPanel labelPanel = new JPanel();
        labelPanel.setBorder(new BevelBorder(BevelBorder.LOWERED));
        labelPanel.add(new JLabel("<HTML>"
                + ResourceManager.getResource("About_Info") + "<p>Version "
                + UserHome.CESE_VERSION_MINOR + "</p>"
                + ResourceManager.getResource("About_copyright") 
                + ResourceManager.getResource("About_supplement") 
                + "</HTML>"));
        productPanel.add(labelPanel, BorderLayout.CENTER);

        return productPanel;
    }

    private JEditorPane createReadmePane() {
        try {
            URL base = ClassLoader.getSystemResource("readme.html");
            readmePanel = new JEditorPane(base);
            ((HTMLDocument) readmePanel.getDocument()).setBase(base);
            readmePanel.setContentType("text/html");
            readmePanel.addHyperlinkListener(this);
            readmePanel.setEditable(false);
        } catch (Exception e) {
        }

        return readmePanel;
    }

    private JTable createSystemPane() {
        JTable systemInfo = null;

        String[][] info;
        String cols[] = { ResourceManager.getResource("Name"),
                ResourceManager.getResource("Value")};

        try {
            info = new String[6][2];

            info[0][0] = "Operation System";
            info[0][1] = System.getProperty("os.name", "unknown") + " "
                    + System.getProperty("os.version", "unknown") + " "
                    + System.getProperty("os.arch", "unknown");

            info[1][0] = "Java Version";
            info[1][1] = System.getProperty("java.vm.name", "unknown") + " "
                    + System.getProperty("java.vm.version", "");

            info[2][0] = "Java Vendor";
            info[2][1] = System.getProperty("java.vendor", "unknown");

            info[3][0] = "Java Home";
            info[3][1] = System.getProperty("java.home", "unknown");

            info[4][0] = "User Home";
            info[4][1] = System.getProperty("user.home", "unknown");

            info[5][0] = "Work Dir";
            info[5][1] = System.getProperty("user.dir", "unknown");

            systemInfo = new JTable(info, cols);
        } catch (Exception e) {
        }

        return systemInfo;
    }

    public void hyperlinkUpdate(HyperlinkEvent event) {
        if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            try {
                readmePanel.setPage(event.getURL());
            } catch (IOException ioe) {
            }
        }
    }
}