/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese.actions;

import java.awt.event.ActionEvent;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;

import com.simulogic.cese.ModelManager;
import com.simulogic.cese.ResourceManager;
import com.simulogic.utils.ActionSupport;
import com.simulogic.utils.ExceptionDialog;

/**
 * SaveParams action implementation
 */
public class SaveParamsAction extends ActionSupport implements
		java.beans.PropertyChangeListener {


	private ModelManager manager;

	/** 
	 * Creates new SaveParamsAction  
     *  
     * @param m model manager
     */
	public SaveParamsAction(ModelManager m) {
		this();
		manager = m;
		manager.addPropertyChangeListener(this);
	}

	/** Creates new SaveParamsAction */
	public SaveParamsAction() {
		super(ResourceManager.getResource("Save_Parameters"),
				ResourceManager.getResource("Save_parameters"), ResourceManager
						.getResource("save_params_image"));
	}

	public void actionPerformed(ActionEvent evt) {
		ObjectOutputStream out = null;

		if (manager.getModelInfo().getHistory().getParametersFile() != null) {
			try {
				out = new ObjectOutputStream(new FileOutputStream(manager
						.getModelInfo().getHistory().getParametersFile()
						.getPath()));

				manager.getModelInfo().getHistory().setTreeSelection(
						manager.getExportTreeSelection());
		
				out.writeObject(manager.getModelInfo().getModel());
				
				manager.getModelInfo().setModified(false);
				manager.changeModel();
			} catch (Exception e) {
				ExceptionDialog.showExceptionDialog(ResourceManager
						.getResource("Error"), ResourceManager
						.getResource("Error_saving_parameters_file."), e);
			} finally {
				try {
					if (out != null)
						out.close();
				} catch (Exception ex) {
				}
			}
		} else {
			SaveParamsAsAction saveAs = new SaveParamsAsAction(manager);
			saveAs.actionPerformed(evt);
		}
	}

	public void propertyChange(java.beans.PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals("model_control"))
			this.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
	}
}