/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese.properties;

/**
 * This class contains attributes of the variable that can be clamped
 */
public class ClampAttribute {

    private String variable;
    private boolean clamped;

    /**
     * Creates new ClampAttribute
     */
    public ClampAttribute() {
        this("", false);
    }

    /**
     * Creates new ClampAttribute
     * 
     * @param var name of the variable
     * @param cl true if variable is clamped
     */
    public ClampAttribute(String var, boolean cl) {
        this.variable = var;
        this.clamped = cl;
    }

    /**
     * @return boolean
     */
    public boolean isClamped() {
        return clamped;
    }

    /**
     * @return String
     */
    public String getVariable() {
        return variable;
    }

    /**
     * Sets the clamped.
     * 
     * @param clamped The clamped to set
     */
    public void setClamped(boolean clamped) {
        this.clamped = clamped;
    }

    public String toString() {
        if (clamped) {
            return "[X]";
        } else {
            return "[ ]";
        }
    }

}