/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese.signals;

/**
 * This is a class for creating a ramp signal
 * <p>
 * 3 parameters for ramp signal:
 * <ol>
 * <li>is an initial amplitude
 * <li>is an final amplitude
 * <li>is a duration
 * </ol>
 */
public class RampSignal extends AbstractSignal {

    /**
     * Creates new ramp signal
     */
    public RampSignal() {
        maxParams = 3;
    }

    /**
     * Builds the signal array
     * 
     * @param dt current sampling step, i.e. distance between 2 data points
     * @return resulting array with signal data
     */
    public double[] buildSignal(double dt) {
        double level1, level2, duration;
        double du;
        double[] result = null;

        try {
            level1 = ((Double) parameters.get(0)).doubleValue();
            level2 = ((Double) parameters.get(1)).doubleValue();
            duration = ((Double) parameters.get(2)).doubleValue();
            if (duration <= 0) return result;

            duration = duration / dt;
            int steps = (int) Math.round(duration);
            result = new double[steps];
            int count = 0;

            du = (level2 - level1) / duration;

            while (count < steps) {
                result[count] = level1;
                level1 += du;
                count++;
            }
        } catch (Exception e) {
        }
        return result;
    }

}