/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import java.awt.Color;

import javax.swing.Box;

/**
 * Generates dialog box for changing view properties
 */
public class ViewProperties extends javax.swing.JDialog {

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    private DataView view;

    /**
     * Default constructor
     * 
     * @param parent parent frame
     * @param v view
     */
    public ViewProperties(java.awt.Frame parent, DataView v) {
        super(parent, true);
        initComponents();
        setView(v);

        pack();
        setResizable(false);
        setLocationRelativeTo(getOwner());
    }

    /**
     * Returns status of this dialog
     * 
     * @return the return status of this dialog - one of RET_OK or RET_CANCEL
     */
    public int getReturnStatus() {
        return returnStatus;
    }

    private void initComponents() {
        viewBackgroundColorEditor = new javax.swing.beaninfo.SwingColorEditor();
        viewScaleColorEditor = new javax.swing.beaninfo.SwingColorEditor();
        viewGridColorEditor = new javax.swing.beaninfo.SwingColorEditor();
        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        viewPanel = new javax.swing.JTabbedPane();
        propertyPanel = new javax.swing.JPanel();
        backgroundPanel = new javax.swing.JPanel();
        backgroundLabel = new javax.swing.JLabel();
        gridColorPanel = new javax.swing.JPanel();
        gridColorLabel = new javax.swing.JLabel();
        scaleColorPanel = new javax.swing.JPanel();
        scaleColorLabel = new javax.swing.JLabel();
        scalePanel = new javax.swing.JPanel();
        xPanel = new javax.swing.JPanel();
        xminPanel = new javax.swing.JPanel();
        xminLabel = new javax.swing.JLabel();
        xminField = new com.simulogic.utils.DoubleField();
        xmaxPanel = new javax.swing.JPanel();
        xmaxLabel = new javax.swing.JLabel();
        xmaxField = new com.simulogic.utils.DoubleField();
        yPanel = new javax.swing.JPanel();
        yminPanel = new javax.swing.JPanel();
        yminLabel = new javax.swing.JLabel();
        yminField = new com.simulogic.utils.DoubleField();
        ymaxPanel = new javax.swing.JPanel();
        ymaxLabel = new javax.swing.JLabel();
        ymaxField = new com.simulogic.utils.DoubleField();

        java.awt.GridBagConstraints gridBagConstraints;

        setTitle(ResourceManager.getResource("View_Properties"));
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {

            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog();
            }
        });

        buttonPanel
                .setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        okButton.setText(ResourceManager.getResource("OK"));
        okButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed();
            }
        });

        buttonPanel.add(okButton);

        cancelButton.setText(ResourceManager.getResource("Cancel"));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed();
            }
        });

        buttonPanel.add(cancelButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        propertyPanel.setLayout(new javax.swing.BoxLayout(propertyPanel,
                javax.swing.BoxLayout.Y_AXIS));

        propertyPanel.setBorder(new javax.swing.border.EmptyBorder(
                new java.awt.Insets(4, 4, 4, 4)));
        backgroundPanel.setLayout(new javax.swing.BoxLayout(backgroundPanel,
                javax.swing.BoxLayout.X_AXIS));

        backgroundLabel.setText(ResourceManager
                .getResource("Background_Color_"));
        backgroundLabel.setDisplayedMnemonic('b');
        backgroundLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        backgroundLabel
                .setLabelFor(viewBackgroundColorEditor.getCustomEditor());
        backgroundPanel.add(backgroundLabel);

        propertyPanel.add(backgroundPanel);

        gridColorPanel.setLayout(new javax.swing.BoxLayout(gridColorPanel,
                javax.swing.BoxLayout.X_AXIS));

        gridColorLabel.setText(ResourceManager.getResource("Grid_Color_"));
        gridColorLabel.setDisplayedMnemonic('g');
        gridColorLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        gridColorLabel.setLabelFor(viewGridColorEditor.getCustomEditor());
        gridColorPanel.add(gridColorLabel);

        propertyPanel.add(gridColorPanel);

        scaleColorPanel.setLayout(new javax.swing.BoxLayout(scaleColorPanel,
                javax.swing.BoxLayout.X_AXIS));

        scaleColorLabel.setText(ResourceManager.getResource("Scale_Color_"));
        scaleColorLabel.setDisplayedMnemonic('s');
        scaleColorLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        scaleColorLabel.setLabelFor(viewScaleColorEditor.getCustomEditor());
        scaleColorPanel.add(scaleColorLabel);

        propertyPanel.add(scaleColorPanel);

        viewPanel.addTab("Visual", propertyPanel);

        scalePanel.setLayout(new java.awt.GridBagLayout());

        scalePanel.setBorder(new javax.swing.border.EmptyBorder(
                new java.awt.Insets(4, 4, 4, 4)));
        xPanel.setLayout(new java.awt.GridLayout(1, 2, 5, 0));

        xminLabel.setText(ResourceManager.getResource("Min_x_"));
        xminLabel.setDisplayedMnemonic('x');
        xminLabel.setLabelFor(xminField);
        xminLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        xminPanel.add(xminLabel);

        xminPanel.add(xminField);

        xPanel.add(xminPanel);

        xmaxLabel.setText(ResourceManager.getResource("Max_x_"));
        xmaxLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        xmaxPanel.add(xmaxLabel);

        xmaxPanel.add(xmaxField);

        xPanel.add(xmaxPanel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        scalePanel.add(xPanel, gridBagConstraints);

        yPanel.setLayout(new java.awt.GridLayout(1, 2, 5, 0));

        yminLabel.setText(ResourceManager.getResource("Min_y_"));
        yminLabel.setDisplayedMnemonic('y');
        yminLabel.setLabelFor(yminField);
        yminLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        yminPanel.add(yminLabel);

        yminPanel.add(yminField);

        yPanel.add(yminPanel);

        ymaxLabel.setText(ResourceManager.getResource("Max_y_"));
        ymaxLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        ymaxPanel.add(ymaxLabel);

        ymaxPanel.add(ymaxField);

        yPanel.add(ymaxPanel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        scalePanel.add(yPanel, gridBagConstraints);

        viewPanel.addTab("Scale", scalePanel);

        backgroundPanel.add(Box.createHorizontalGlue());
        backgroundPanel.add(viewBackgroundColorEditor.getCustomEditor());
        scaleColorPanel.add(Box.createHorizontalGlue());
        scaleColorPanel.add(viewScaleColorEditor.getCustomEditor());
        gridColorPanel.add(Box.createHorizontalGlue());
        gridColorPanel.add(viewGridColorEditor.getCustomEditor());

        getContentPane().add(viewPanel, java.awt.BorderLayout.CENTER);
    }

    private void okButtonActionPerformed() {
        view.setBackground(((Color) viewBackgroundColorEditor.getValue()));
        view.setGridColor(((Color) viewGridColorEditor.getValue()));
        view.setScaleColor(((Color) viewScaleColorEditor.getValue()));

        view.setScale(xminField.getValue(), ymaxField.getValue(), xmaxField
                .getValue()
                - xminField.getValue(), ymaxField.getValue()
                - yminField.getValue());

        doClose(RET_OK);
        view.renderPlot();
    }

    private void cancelButtonActionPerformed() {
        doClose(RET_CANCEL);
    }

    /** 
     * Closes the dialog
     */
    private void closeDialog() {
        doClose(RET_CANCEL);
    }

    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setVisible(false);
        dispose();
     }

    private javax.swing.JPanel scalePanel;
    private javax.swing.JPanel backgroundPanel;
    private javax.swing.JPanel xmaxPanel;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JPanel xminPanel;
    private javax.swing.JPanel scaleColorPanel;
    private javax.swing.beaninfo.SwingColorEditor viewBackgroundColorEditor;
    private javax.swing.JLabel backgroundLabel;
    private javax.swing.JPanel ymaxPanel;
    private javax.swing.JLabel xmaxLabel;
    private javax.swing.JPanel yPanel;
    private javax.swing.JPanel xPanel;
    private javax.swing.JButton okButton;
    private javax.swing.beaninfo.SwingColorEditor viewGridColorEditor;
    private javax.swing.JButton cancelButton;
    private javax.swing.JPanel yminPanel;
    private com.simulogic.utils.DoubleField xmaxField;
    private javax.swing.JLabel xminLabel;
    private javax.swing.JLabel scaleColorLabel;
    private javax.swing.JPanel gridColorPanel;
    private com.simulogic.utils.DoubleField xminField;
    private javax.swing.JLabel ymaxLabel;
    private javax.swing.JPanel propertyPanel;
    private javax.swing.beaninfo.SwingColorEditor viewScaleColorEditor;
    private com.simulogic.utils.DoubleField ymaxField;
    private javax.swing.JLabel yminLabel;
    private javax.swing.JLabel gridColorLabel;
    private javax.swing.JTabbedPane viewPanel;
    private com.simulogic.utils.DoubleField yminField;

    private int returnStatus = RET_CANCEL;

    /**
     * Sets new view
     * 
     * @param v new view
     */
    public void setView(DataView v) {
        this.view = v;
        viewBackgroundColorEditor.setValue(view.getBackground());
        viewGridColorEditor.setValue(view.getGridColor());
        viewScaleColorEditor.setValue(view.getScaleColor());

        xminField.setValue(view.getScale().getX());
        xmaxField.setValue(view.getScale().getX()
                + view.getScale().getWidth());
        yminField.setValue(view.getScale().getY()
                - view.getScale().getHeight());
        ymaxField.setValue(view.getScale().getY());

        getRootPane().setDefaultButton(okButton);
    }
}