/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this library; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.dataview.actions;

import com.simulogic.dataview.ResourceManager;
import com.simulogic.utils.ExceptionDialog;

/**
 * PrintRes action implementation
 */
public class PrintResAction extends com.simulogic.utils.ActionSupport implements
        java.beans.PropertyChangeListener {

    private com.simulogic.dataview.DataDesktop desktop;

    /**
     * Creates new action
     * 
     * @param d desktop
     */
    public PrintResAction(com.simulogic.dataview.DataDesktop d) {
        this();
        desktop = d;
        desktop.addPropertyChangeListener(this);
    }

    /**
     * Creates new action
     */
    public PrintResAction() {
        super(ResourceManager.getResource("Print_Results..."), ResourceManager
                .getResource("Print_results"), ResourceManager
                .getResource("print_image"));
    }

    /**
     * Action interface implementation
     * 
     * @param evt event
     */
    public void actionPerformed(java.awt.event.ActionEvent evt) {
		com.simulogic.utils.SwingWorker worker = new com.simulogic.utils.SwingWorker() {
			Exception ex = null;

			public Object construct() {
				try {
					ex = null;
					desktop.getPreview().showPreview();
					desktop.getPreview().doPrint();
				} catch (Exception e) {
					ex = e;
				}
				return ResourceManager.getResource("Done");
			}

			public void finished() {
				desktop.getTasks().removeTask(
						ResourceManager.getResource("Print_results"));

				if (ex != null)
					ExceptionDialog.showExceptionDialog(ResourceManager
							.getResource("Error"), ResourceManager
							.getResource("Error_printing_results."), ex);
			}
		};

		if (desktop.getPreview().showPrintDialog()) {
			desktop.getTasks().addTask(
					ResourceManager.getResource("Print_results"));
			worker.start();
		}
	}

    /**
	 * Reflects change in the property (disables/enables) action
	 * 
	 * @param evt
	 *            event
	 */
    public void propertyChange(java.beans.PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals("frame_control"))
            this.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
    }
}