/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.export;

import java.io.IOException;

import com.simulogic.dataview.DataChannel;
import com.simulogic.dataview.DataGroup;
import com.simulogic.dataview.DataStorage;
import com.simulogic.dataview.ResourceManager;
import com.simulogic.dataview.ViewContent;

/**
 * Class that exports data to ASCII format
 */
public class AsciiExport {

	private com.simulogic.dataview.ViewContent view;

	private String delimiter;

	private String format;

	/**
	 * Default constructor
	 * 
	 * @param v
	 *            view conent
	 */
	public AsciiExport(ViewContent v) {
		view = v;

		format = "0.##########";
		delimiter = String.valueOf('\t');
	}

	/**
	 * Exports the content of <code>DataStorage</code> to ASCII file
	 * 
	 * @param fileName
	 *            name of the file to export data
	 * @throws IOException
	 *             if i/o problem occurs
	 */
	public synchronized void export(String fileName) throws java.io.IOException {
		java.io.PrintWriter datExp = new java.io.PrintWriter(
				new java.io.FileWriter(fileName));

		if ((view.getTabs().indexOfTab(ResourceManager.getResource("Cursors")) == view
				.getTabs().getSelectedIndex())
				&& (view.getTabs().indexOfTab(
						ResourceManager.getResource("Cursors")) != -1)) {
			datExp.print(exportDataASCII(view.getView().getCursors()
					.getCursorsStorage()));
		} else {
			datExp.print(exportDataASCII(view.getView().getStorage()));
		}

		datExp.flush();
		datExp.close();
	}

	/**
	 * Exports data to ASCII
	 * 
	 * Only channels with attribute NORMAL_CHANNEL (default) will be exported
	 * 
	 * @param storage data storage to export
	 * @return String containing formatted ASCII codes for data
	 */
	public String exportDataASCII(DataStorage storage) {
		double recs = 0;

		DataGroup group = null;
		DataChannel channel = null;

		StringBuffer datOut = new StringBuffer();
		java.text.DecimalFormat datFormat = new java.text.DecimalFormat(format);

		try {
			datOut.append("# ");

			for (int i = 0; i < storage.getGroupsSize(); i++) {
				for (int j = 0; j < storage.getChannelsSize(i); j++) {
					channel = storage.getChannel(i, j);
					if (channel.getAttribute().isNormal()) {
						recs = Math.max(recs, channel.size());
						datOut.append(channel.getName() + delimiter);
					}
				}
			}
			datOut.append('\n');

			for (int k = 0; k < recs; k++) {
				for (int i = 0; i < storage.getGroupsSize(); i++) {
					group = storage.getGroup(i);
					for (int j = 0; j < storage.getChannelsSize(i); j++) {
						channel = storage.getChannel(i, j);
						if (channel.getAttribute().isNormal()) {
							if (k < channel.size()) {
								datOut.append(datFormat.format(channel
										.getData(k))
										+ delimiter);
							} else {
								datOut.append(" " + delimiter);
							}
						}
					}
				}
				datOut.append('\n');
			}
		} catch (Exception e) {
		}

		return datOut.toString();
	}

	/**
	 * Getter for property delimiter
	 * 
	 * @return Value of property delimiter
	 */
	public String getDelimiter() {
		return delimiter;
	}

	/**
	 * Setter for property delimiter
	 * 
	 * @param delimiter
	 *            New value of property delimiter
	 */
	public void setDelimiter(String delimiter) {
		this.delimiter = delimiter;
	}

	/**
	 * Getter for property format.
	 * 
	 * @return Value of property format.
	 */
	public String getFormat() {
		return format;
	}

	/**
	 * Setter for property format.
	 * 
	 * @param format
	 *            New value of property format.
	 */
	public void setFormat(String format) {
		this.format = format;
	}
}
