/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.legend;

import javax.swing.JPanel;
import javax.swing.UIManager;

import com.simulogic.dataview.DataView;

/**
 * Renderer for the legend tree
 */
public class LegendTreeRenderer implements javax.swing.tree.TreeCellRenderer {

	private LegendTreeEditor editor;

	private javax.swing.border.Border selectedBorder;

	private javax.swing.border.Border emptyBorder;

	private javax.swing.JPanel renderingContainer;

	/**
	 * Creates new renderer for the legend tree
	 * 
	 * @param v
	 *            data view
	 */
	public LegendTreeRenderer(DataView v) {
		selectedBorder = javax.swing.BorderFactory.createLineBorder(UIManager
				.getColor("Tree.selectionBorderColor"), 1);
		emptyBorder = javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1);

		editor = new LegendTreeEditor(v);
		editor.setRenderable(true);
	}

	/**
	 * Returns rendering component
	 * 
	 * @param tree
	 *            export tree
	 * @param value
	 *            current value
	 * @param selection
	 *            true if value is selected
	 * @param expanded
	 *            true if value is expanded
	 * @param leaf
	 *            true if value is leaf
	 * @param row
	 *            current row number
	 * @param hasFocus
	 *            true if value has focus
	 * @return rendering component
	 */
	public java.awt.Component getTreeCellRendererComponent(
			javax.swing.JTree tree, Object value, boolean selection,
			boolean expanded, boolean leaf, int row, boolean hasFocus) {

		renderingContainer = (JPanel) editor.getTreeCellEditorComponent(tree,
				value, selection, expanded, leaf, row);

		renderingContainer.setOpaque(true);

		if (selection) {
			renderingContainer.setBackground(UIManager
					.getColor("Tree.selectionBackground"));
		} else {
			renderingContainer.setBackground(UIManager
					.getColor("Tree.textBackground"));
		}

		if (hasFocus) {
			renderingContainer.setBorder(selectedBorder);
		} else {
			renderingContainer.setBorder(emptyBorder);
		}

		return renderingContainer;
	}

	public void setPrintMode(boolean p) {
		editor.setPrintMode(p);
	}

}