/* 
 * Copyright (C) 2000 Victor Smirnov. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include  the following  acknowledgment:  "This product includes  software
 *    developed by Victor Smirnov (http://www.simulogic.com/)." Alternately, this
 *    acknowledgment may  appear in  the software itself,  if and wherever such
 *    third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE  IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING,  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED. IN NO  EVENT SHALL VICTOR
 * SMIRNOV OR  simulogic OR ITS CONTRIBUTORS BE LIABLE  FOR ANY  DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA,
 * OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON ANY  THEORY OF
 * LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT (INCLUDING  NE-
 * GLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.simulogic.utils;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * Basic class for actions with support for setting action properties.
 * 
 * @author <a href="mailto:victor@uwc.ru">Victor Smirnov </a>
 * @version $Revision: 1.1 $ $Date: 2007/02/16 20:37:43 $
 */
public class ActionSupport extends AbstractAction {

    /**
     * Create new ActionSupport.
     * 
     * @param name action name.
     */
    public ActionSupport(String name) {
        super(name);
    }

    /**
     * Create new ActionSupport.
     * 
     * @param name action name.
     * @param dsc short description.
     */
    public ActionSupport(String name, String dsc) {
        super(name);
        setShortDescription(dsc);
    }

    /**
     * Create new ActionSupport.
     * 
     * @param name action name.
     * @param dsc short description.
     * @param icon small icon name.
     */
    public ActionSupport(String name, String dsc, String icon) {
        super(name);
        setShortDescription(dsc);
        setSmallIcon(icon);
    }

    /**
     * Get action name.
     * 
     * @return action name.
     */
    public String getName() {
        return (String) getValue(Action.NAME);
    }

    /**
     * Set action name.
     * 
     * @param name new action name.
     */
    public void setName(String name) {
        putValue(Action.NAME, name);
    }

    /**
     * Get short description for the action, it is used for tooltip text.
     * 
     * @return short descripition of an action.
     */
    public String getShortDescription() {
        return (String) getValue(Action.SHORT_DESCRIPTION);
    }

    /**
     * Set short description for the action.
     * 
     * @param dsc short description.
     */
    public void setShortDescription(String dsc) {
        putValue(Action.SHORT_DESCRIPTION, dsc);
    }

    /**
     * Get action small icon, used in menus and buttons.
     * 
     * @return action small icon.
     */
    public Icon getSmallIcon() {
        return (Icon) getValue(Action.SMALL_ICON);
    }

    /**
     * Set action small icon.
     * 
     * @param icon icon for the action.
     */
    public void setSmallIcon(Icon icon) {
        putValue(Action.SMALL_ICON, icon);
    }

    /**
     * Set action small icon.
     * 
     * @param icon icon name.
     */
    public void setSmallIcon(String icon) {
        setSmallIcon(new ImageIcon(this.getClass().getResource(icon)));
    }

    /**
     * Performs the action.
     * 
     * @param e event
     */
    public void actionPerformed(ActionEvent e) {
        System.out.println("Action event [" + e.getActionCommand()
                + "] recieved by Action [" + getName());
        System.out.println("Event source " + e.getSource());
    }
}