/*
 * Author : Tom Yarker, 22695 Dorchester Drive, Geneseo, IL 61254 Phone: (309)
 * 944-5955, E-Mail: tyarker@netexpress.net Purpose: Expression evaluator.
 * History: 08-29-97 Started 08-11-98 Prevented multiply of explicit constants
 */

package com.simulogic.utils;

import java.awt.Point;
import java.util.Vector;

public class TYParse {

    // Next location
    private int next;
    // List of variable names and values
    private Vector vnam, vval;
    // List of operation codes and values
    private Vector tcod, tval;
    // Functions and their opcodes
    private static final String[] fnam = { "+", "-", "*", "/", "^", "(", ")",
            "[", "]", "{", "}", "sin", "cos", "tan", "asin", "acos", "atan",
            "abs", "sqrt", "root", "log10", "log", "exp", "alog10", "alog",
            "int", "nint"};
    private static final int[] fcod = { 1, 2, 3, 4, 5, 6, 7, 6, 7, 6, 7, 8, 9,
            10, 11, 12, 13, 14, 15, 15, 19, 17, 16, 18, 16, 20, 21};

    //-----------------------------------------------------

    // Constructor
    public TYParse() {
        next = 0;
        vnam = new Vector(4, 2);
        vval = new Vector(4, 2);
        // Set up constants
        addVar("e", Math.E);
        addVar("pi", Math.PI);
    }

    //-----------------------------------------------------
    // From this point on, routines are in alphabetic order
    //-----------------------------------------------------

    // Adds a token to the list
    // 1st argument is the opcode
    // 2nd argument is the value
    // 3rd argument is the location
    private void addTok(int cd, double vl, int lo) {
        tcod.addElement(new Point(cd, lo));
        tval.addElement(new Double(vl));
    }

    //-----------------------------------------------------

    // Adds a variable to the list
    // 1st argument is the name
    // 2nd argument is the value
    private void addVar(String nm, double vl) {
        vnam.addElement(nm);
        vval.addElement(new Double(vl));
    }

    //-----------------------------------------------------

    // Gets list of variable names and values
    public String getList() {
        sortVar();
        String st = "";
        for (int i = 0; i < vnam.size(); i = i + 1)
            st = st + getVarN(i) + " = " + getVarV(i) + "\n";
        return st;
    }

    //-----------------------------------------------------

    // Parses a string to obtain a variable name
    // 1st argument is the string (starts with non-space)
    // 2nd argument is index of first character of interest
    private String getName(String st, int nx) {
        String vn = "";
        boolean gl = false;
        // Loop thru characters
        int nc = st.length();
        for (next = nx; next < nc; next = next + 1) {
            char ch = st.charAt(next);
            if (Character.isLetter(ch)) {
                vn = vn + ch;
                gl = true;
                continue;
            }
            if (Character.isDigit(ch) && gl) {
                vn = vn + ch;
                continue;
            }
            break;
        }
        return vn;
    }

    //-----------------------------------------------------

    // Returns next location in string
    public int getNext() {
        return next;
    }

    //-----------------------------------------------------

    // Parses a string to obtain a value
    // 1st argument is the string
    public double getNumb(String st) {
        return getNumb(st, 0);
    }

    //-----------------------------------------------------

    // Parses a string to obtain a value
    // 1st argument is the string
    // 2nd argument is index of first character of interest
    private double getNumb(String st, int nx) {
        // Establish mantissa and exponent
        double am = 0;
        double dm = 0;
        double ae = 0;
        int sm = 1;
        int se = 1;
        // White, sign, decimal OK
        boolean wk = true;
        boolean sk = true;
        boolean dk = true;
        // But not an exponent yet
        boolean ek = false;
        // No mantissa or exponent
        boolean gm = false;
        boolean ge = false;
        // Loop thru characters
        int nc = st.length();
        for (next = nx; next < nc; next = next + 1) {
            char ch = st.charAt(next);
            if (wk) {
                if (Character.isWhitespace(ch)) continue;
                wk = false;
            }
            // Check for sign
            if (sk) {
                sk = false;
                if (ch == '+') continue;
                if (ch == '-') {
                    if (ge)
                        se = -1;
                    else
                        sm = -1;
                    continue;
                }
            }
            // Check for numeral
            int iv = Character.digit(ch, 10);
            if (iv >= 0) {
                if (ge)
                    ae = ae * 10 + iv;
                else {
                    am = am * 10 + iv;
                    dm = dm * 10;
                    gm = true;
                    ek = true;
                }
                continue;
            }
            // Check for a decimal
            if (dk && (ch == '.' || ch == ',')) {
                dk = false;
                dm = 1;
                continue;
            }
            // Check for an exponent
            if (ek && (ch == 'E' || ch == 'D' || ch == 'e' || ch == 'd')) {
                ek = false;
                dk = false;
                sk = true;
                ge = true;
                continue;
            }
            break;
        }
        // Assemble the value
        if (gm) {
            am = sm * am;
            if (dm != 0) am = am / dm;
            if (ge) {
                ae = se * ae;
                am = am * Math.pow(10., ae);
            }
        } else
            next = nx;
        return am;
    }

    //-----------------------------------------------------

    // Get a token's opcode
    // 1st argument is index of the token
    private int getTokC(int it) {
        return ((Point) tcod.elementAt(it)).x & 31;
    }

    //-----------------------------------------------------

    // Get a token's location
    // 1st argument is index of the token
    private int getTokL(int it) {
        return ((Point) tcod.elementAt(it)).y;
    }

    //-----------------------------------------------------

    // Get a token's state
    // 1st argument is index of the token
    private boolean getTokS(int it) {
        return ((Point) tcod.elementAt(it)).x > 31;
    }

    //-----------------------------------------------------

    // Get a token's value
    // 1st argument is index of the token
    private double getTokV(int it) {
        return ((Double) tval.elementAt(it)).doubleValue();
    }

    //-----------------------------------------------------

    // Evaluates an expression
    // 1st argument is the expression
    public double getValue(String st) {
        next = 0;
        int nc = st.length();
        // Get index of expression
        int le = st.indexOf("=") + 1;
        if (le == nc) return 0;
        // Parse it into opcodes & values
        parseIt(st, le);
        int ic = tcod.size();
        if (ic == 0 || next < nc) return 0;
        int i1 = 0;
        int i2 = 0;
        boolean ae;
        double av, bv;
        double rd = Math.PI / 180;
        double dr = 180 / Math.PI;
        while (tcod.size() > 1) {
            // Look for "+value" or "-value"
            int i3 = 0;
            ae = false;
            while (!ae) {
                i2 = indexOf(1, 0, -1, i3);
                i1 = indexOf(2, 0, -1, i3);
                if (i2 < 0) i2 = nc;
                if (i1 < 0) i1 = nc;
                i1 = Math.min(i1, i2);
                if (i1 == nc) break;
                if (i1 == 0)
                    ae = true;
                else {
                    ic = getTokC(i1 - 1);
                    if (ic != 0 && ic != 7) ae = true;
                }
                i3 = i1 + 2;
            }
            // Remove the sign
            if (ae) {
                remTok(i1);
                if (i1 != i2) setTokV(-getTokV(i1), i1);
                continue;
            }
            // Look for "(value)" and drop parentheses
            i1 = indexOf(6, 0, 7, 0);
            if (i1 >= 0) {
                remTok(i1);
                setTokS(i1);
                remTok(i1 + 1);
                continue;
            }
            // Look for "function value"
            ic = -1;
            for (int i = 8; i < 22; i = i + 1) {
                i1 = indexOf(i, 0, -1, 0);
                if (i1 < 0) continue;
                ic = i;
                break;
            }
            // Remove the function & get its argument
            if (ic >= 0) {
                remTok(i1);
                av = getTokV(i1);
                // Branch on function code
                ae = false;
                switch (ic) {
                case 8:
                    av = Math.sin(av * rd);
                    break;
                case 9:
                    av = Math.cos(av * rd);
                    break;
                case 10:
                    bv = Math.cos(av * rd);
                    if (bv == 0)
                        ae = true;
                    else
                        av = Math.sin(av * rd) / bv;
                    break;
                case 11:
                    bv = Math.abs(av);
                    if (bv > 1)
                        ae = true;
                    else
                        av = Math.asin(av) * dr;
                    break;
                case 12:
                    bv = Math.abs(av);
                    if (bv > 1)
                        ae = true;
                    else
                        av = Math.acos(av) * dr;
                    break;
                case 13:
                    av = Math.atan(av) * dr;
                    break;
                case 14:
                    av = Math.abs(av);
                    break;
                case 15:
                    if (av < 0)
                        ae = true;
                    else
                        av = Math.sqrt(av);
                    break;
                case 16:
                    av = Math.exp(av);
                    break;
                case 17:
                    if (av <= 0)
                        ae = true;
                    else
                        av = Math.log(av);
                    break;
                case 18:
                    av = Math.pow(10., av);
                    break;
                case 19:
                    if (av <= 0)
                        ae = true;
                    else
                        av = Math.log(av) / Math.log(10.);
                    break;
                case 20:
                    av = (long) av;
                    break;
                case 21:
                    av = Math.round(av);
                    break;
                }
                // Check for an arithmetic error
                if (ae) {
                    next = getTokL(i1);
                    break;
                }
                // Update the value
                setTokV(av, i1);
                continue;
            }
            // Look for "value ^ value"
            i1 = indexOf(0, 5, 0, 0);
            if (i1 >= 0) {
                av = getTokV(i1);
                remTok(i1);
                remTok(i1);
                bv = getTokV(i1);
                if (av == 0 && bv <= 0) ae = true;
                if (av < 0 && bv != (int) bv) ae = true;
                if (ae) {
                    next = getTokL(i1);
                    break;
                }
                setTokV(Math.pow(av, bv), i1);
                continue;
            }
            // Find first multiplication or division
            i3 = indexOf(0, 0, -1, 0);
            i2 = indexOf(0, 3, 0, 0);
            i1 = indexOf(0, 4, 0, 0);
            if (i3 < 0) i3 = nc;
            if (i2 < 0) i2 = nc;
            if (i1 < 0) i1 = nc;
            i1 = Math.min(i1, i2);
            i1 = Math.min(i1, i3);
            if (i1 < nc) {
                av = getTokV(i1);
                // Handle "value value"
                if (i1 == i3) {
                    if (getTokS(i1) || getTokS(i1 + 1)) {
                        remTok(i1);
                        setTokV(av * getTokV(i1), i1);
                        continue;
                    } else {
                        next = getTokL(i1 + 1);
                        break;
                    }
                }
                // Check for pre division or power, or post power
                bv = getTokV(i1 + 2);
                if (i1 > 0) {
                    ic = getTokC(i1 - 1);
                    if (ic == 4 || ic == 5) ae = true;
                }
                if (i1 + 3 < tcod.size()) {
                    ic = getTokC(i1 + 3);
                    if (ic == 4 || ic == 5) ae = true;
                }
                // Perform the multiplication or division
                if (!ae) {
                    remTok(i1);
                    remTok(i1);
                    if (i1 == i2)
                        av = av * bv;
                    else {
                        if (bv == 0) {
                            next = getTokL(i1);
                            break;
                        }
                        av = av / bv;
                    }
                    setTokV(av, i1);
                    continue;
                }
            }
            // Find first addition or subtraction
            i3 = 0;
            ae = false;
            while (!ae) {
                i2 = indexOf(0, 1, 0, i3);
                i1 = indexOf(0, 2, 0, i3);
                if (i2 < 0) i2 = nc;
                if (i1 < 0) i1 = nc;
                i1 = Math.min(i1, i2);
                if (i1 == nc) break;
                ae = true;
                // Check for post other than same or right paren
                if (i1 + 3 < tcod.size()) {
                    ic = getTokC(i1 + 3);
                    if (ic > 2 && ic != 7) ae = false;
                }
                i3 = i1 + 2;
            }
            // Perform the addition or subtraction
            if (ae) {
                av = getTokV(i1);
                remTok(i1);
                remTok(i1);
                bv = getTokV(i1);
                if (i1 != i2) bv = -bv;
                setTokV(av + bv, i1);
                continue;
            }
            // No sense encountered
            i1 = tcod.size() - 1;
            next = getTokL(i1);
            break;
        }
        if (tcod.size() > 1) return 0;
        // One opcode left, check for a value
        ic = getTokC(0);
        if (ic != 0) {
            next = getTokL(0);
            return 0;
        }
        av = getTokV(0);
        // Define variable if given
        if (le > 0) {
            String vn = getName(st, 0);
            next = nc;
            if (!vn.equals("")) {
                i1 = indexOf(vn);
                if (i1 < 0)
                    addVar(vn, av);
                else
                    setVarV(av, i1);
            }
        }
        return av;
    }

    //-----------------------------------------------------

    // Get a variable's name
    // 1st argument is index of the variable
    private String getVarN(int iv) {
        return (String) vnam.elementAt(iv);
    }

    //-----------------------------------------------------

    // Get a variable's value
    // 1st argument is index of the variable
    private double getVarV(int iv) {
        return ((Double) vval.elementAt(iv)).doubleValue();
    }

    //-----------------------------------------------------

    // Gets index of variable in list
    // 1st argument is variable name
    // Returns -1 if its not in list
    private int indexOf(String st) {
        for (int i = 0; i < vnam.size(); i = i + 1)
            if (getVarN(i).equalsIgnoreCase(st)) return i;
        return -1;
    }

    //-----------------------------------------------------

    // Gets index of sequence of 3 opcodes
    // 1st argument is 1st opcode
    // 2nd argument is 2nd opcode
    // 3rd argument is 3rd opcode
    //     negative if there isn't one
    // 4th argument is 1st index of interest
    // Returns -1 if sequence not in list
    private int indexOf(int c1, int c2, int c3, int i0) {
        int ns = tcod.size() - 2;
        if (c3 < 0) ns = ns + 1;
        for (int i = i0; i < ns; i = i + 1) {
            int cv = getTokC(i);
            if (c1 != cv) continue;
            cv = getTokC(i + 1);
            if (c2 != cv) continue;
            if (c3 < 0) return i;
            cv = getTokC(i + 2);
            if (c3 == cv) return i;
        }
        return -1;
    }

    //-----------------------------------------------------

    // Parses string into opcodes and values
    // 1st argument is the string
    // 2nd argument is index of first character of interest
    private void parseIt(String st, int nx) {
        int nf = fnam.length;
        tcod = new Vector(8, 4);
        tval = new Vector(8, 4);
        // Skip leading spaces
        next = nx;
        int nc = st.length();
        while (next < nc) {
            char ch = st.charAt(next);
            if (Character.isWhitespace(ch)) {
                next = next + 1;
                continue;
            }
            int iv;
            double vl;
            String vn;
            int lo = next;
            // Look for a known variable
            vn = getName(st, lo);
            if (lo < next) {
                iv = indexOf(vn);
                if (iv < 0)
                    next = lo;
                else {
                    vl = getVarV(iv);
                    addTok(32, vl, next - 1);
                    continue;
                }
            }
            // Look for a function
            iv = Math.min(lo + 6, nc);
            vn = st.substring(lo, iv).toLowerCase();
            iv = -1;
            for (int i = 0; i < nf; i = i + 1) {
                if (vn.startsWith(fnam[i])) {
                    iv = i;
                    break;
                }
            }
            if (iv >= 0) {
                next = lo + fnam[iv].length();
                addTok(fcod[iv], 0, next - 1);
                continue;
            }
            // Look for a numeric value
            vl = getNumb(st, lo);
            if (lo < next) {
                addTok(0, vl, next - 1);
                continue;
            }
            // No sense encountered
            return;
        }
    }

    //-----------------------------------------------------

    // Removes a token from the list
    // 1st argument is index of the token
    private void remTok(int it) {
        tcod.removeElementAt(it);
        tval.removeElementAt(it);
    }

    //-----------------------------------------------------

    // Set a token's status to implicit
    // 1st argument is index of the token
    private void setTokS(int it) {
        Point pt = (Point) tcod.elementAt(it);
        pt.x = (pt.x & 31) + 32;
    }

    //-----------------------------------------------------

    // Set a token's value
    // 1st argument is the value
    // 2nd argument is index of the token
    private void setTokV(double vl, int it) {
        tval.setElementAt(new Double(vl), it);
        setTokS(it);
    }

    //-----------------------------------------------------

    // Set a variable's value
    // 1st argument is the value
    // 2nd argument is index of the variable
    private void setVarV(double vl, int iv) {
        vval.setElementAt(new Double(vl), iv);
    }

    //-----------------------------------------------------

    // Sorts variables alphabetically by name
    private void sortVar() {
        int nl = vnam.size();
        if (nl < 2) return;
        int nn = 1;
        while (nn < nl)
            nn = nn + nn;
        nn = nn / 2;
        int ip = nn;
        String vj, vk;
        double dj, dk;
        while (ip > 0) {
            int iq = nn;
            int id = ip;
            int ir = 0;
            while (true) {
                int nm = nl - id;
                for (int j = 0; j < nm; j = j + 1) {
                    int i = j & ip;
                    if (i != ir) continue;
                    int k = j + id;
                    vj = getVarN(j);
                    vk = getVarN(k);
                    if (vj.compareTo(vk) > 0) {
                        vnam.setElementAt(vk, j);
                        vnam.setElementAt(vj, k);
                        dj = getVarV(j);
                        dk = getVarV(k);
                        setVarV(dk, j);
                        setVarV(dj, k);
                    }
                }
                if (iq == ip) break;
                id = iq - ip;
                iq = iq / 2;
                ir = ip;
            }
            ip = ip / 2;
        }
    }

    //-----------------------------------------------------

}