<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:math="http://www.w3.org/1998/Math/MathML" version="1.0">

<xsl:strip-space elements="*"/>

<xsl:template match="math:vector">
     <xsl:text>new javax.vecmath.GVector(double[]{</xsl:text>
  <xsl:for-each select="./*">
    <xsl:value-of select="translate(.,' ','')" disable-output-escaping = "yes"/>
    <xsl:if test="position()&lt;last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
  </xsl:for-each>
     <xsl:text>})</xsl:text>
</xsl:template>

<xsl:template match="math:matrix">
     <xsl:text>new javax.vecmath.GMatrix(</xsl:text>
     <xsl:value-of select="count(./*)"/>
     <xsl:text>, </xsl:text>
     <xsl:value-of select="count(./*[1]/*)"/>
     <xsl:text>, double[]{</xsl:text>
  <xsl:for-each select="./*">
    <xsl:call-template name="matrixrow"/>
    <xsl:if test="position()&lt;last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
  </xsl:for-each>
     <xsl:text>})</xsl:text>
</xsl:template>

<xsl:template match="math:matrixrow" name="matrixrow">
  <xsl:for-each select="./*">
    <xsl:value-of select="translate(.,' ','')" disable-output-escaping = "yes"/>
    <xsl:if test="position()&lt;last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<!-- not perfect yet need to learn keys? and id? so the xsl can discover the
current dimensions of the matrix.  There is also the problem of the matrix
being defined later.-->
<xsl:template match="math:determinant">
  <xsl:variable name="matrix" select="translate(parent::math:apply/*[2],' ','')"/>
    <xsl:text>double det=0.0;
    if(</xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getNumCol()==2)
    {
        det=</xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getElement(1,1)*</xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getElement(2,2)-</xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getElement(1,2)*</xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getElement(2,1);
    }
    else if(</xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getNumCol()==3)
    {
        genericMatrix3d=new javax.vecmath.Matrix3d();
        </xsl:text><xsl:value-of select="$matrix"/><xsl:text>.get(genericMatrix3d);
        det=genericMatrix3d.determinant();
    }
    else if(</xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getNumCol()==4)
    {
        genericMatrix4d=new javax.vecmath.Matrix4d();
        </xsl:text><xsl:value-of select="$matrix"/><xsl:text>.get(genericMatrix4d);
        det=genericMatrix4d.determinant();
    }
    else
    {
        det=determinant(</xsl:text><xsl:value-of select="$matrix"/><xsl:text>);
    }
    </xsl:text>
</xsl:template>

<xsl:template match="math:transpose">
  <xsl:variable name="matrix" select="translate(parent::math:apply/*[2],' ','')"/>
  <xsl:text></xsl:text><xsl:value-of select="$matrix"/><xsl:text>.transpose()</xsl:text>
</xsl:template>

<xsl:template match="math:selector">
  <xsl:variable name="matrix" select="translate(parent::math:apply/*[2],' ','')"/>
  <xsl:variable name="i" select="translate(parent::math:apply/*[3],' ','')"/>
  <xsl:variable name="j" select="translate(parent::math:apply/*[4],' ','')"/>
  <xsl:text></xsl:text><xsl:value-of select="$matrix"/><xsl:text>.getElement(</xsl:text><xsl:value-of select="$i"/><xsl:text>, </xsl:text><xsl:value-of select="$j"/><xsl:text>)</xsl:text>
</xsl:template>

<xsl:template match="math:scalarproduct">
  <xsl:variable name="vector1" select="translate(parent::math:apply/*[2],' ','')"/>
  <xsl:variable name="vector2" select="translate(parent::math:apply/*[3],' ','')"/>
  <xsl:text></xsl:text><xsl:value-of select="$vector1"/><xsl:text>.dot(</xsl:text><xsl:value-of select="$vector2"/><xsl:text>)</xsl:text>
</xsl:template>

<!-- not sure what an outerproduct is-->
<xsl:template match="math:outerproduct">
  <xsl:variable name="vector1" select="translate(parent::math:apply/*[2],' ','')"/>
  <xsl:variable name="vector2" select="translate(parent::math:apply/*[3],' ','')"/>
  <xsl:text></xsl:text><xsl:value-of select="$vector1"/><xsl:text>.dot(</xsl:text><xsl:value-of select="$vector2"/><xsl:text>)</xsl:text>
</xsl:template>

<!-- not completed yet-->
<xsl:template match="math:vectorproduct">
  <xsl:variable name="vector1" select="translate(parent::math:apply/*[2],' ','')"/>
  <xsl:variable name="vector2" select="translate(parent::math:apply/*[3],' ','')"/>
  <xsl:text>magnitude(</xsl:text><xsl:value-of select="$vector1"/><xsl:text>)*magnitude(</xsl:text><xsl:value-of select="$vector2"/><xsl:text>)*Math.sin(</xsl:text><xsl:value-of select="$vector1"/><xsl:text>.angle(</xsl:text><xsl:value-of select="$vector2"/><xsl:text>))</xsl:text>
</xsl:template>

</xsl:stylesheet>
